/**********************************************************************
Copyright (c) 2009 Stefan Seelmann and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


Contributors :
 ...
 ***********************************************************************/

package org.datanucleus.store.ldap.fieldmanager;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Calendar;

import org.datanucleus.exceptions.NucleusException;
import org.datanucleus.store.types.ObjectStringConverter;
import org.datanucleus.util.ClassUtils;

/**
 * Class to handle the conversion between java.util.Calendar and the generalized time syntax as described in RFC 4517.
 */
public class CalendarToGeneralizedTimeStringConverter implements ObjectStringConverter
{

    public Object toObject(String str)
    {
        if (str == null)
        {
            return null;
        }

        try
        {
            // this class cannot be linked at compile time since its dependencies are considered as optional
            Class cls = Class.forName("org.apache.directory.shared.ldap.util.GeneralizedTime");

            Constructor constr = ClassUtils.getConstructorWithArguments(cls, new Class[]{String.class});
            Object instance = constr.newInstance(new Object[]{str});

            Method method = ClassUtils.getMethodForClass(cls, "getCalendar", null);
            Calendar calendar = (Calendar) method.invoke(instance, (Object[]) null);

            // workaround: the GeneralizedTime sets the calendar to non-lenient, we reset it here to its default value
            calendar.setLenient(true);

            // most LDAP servers only store "century year month day hour minute second"
            // TODO: add extension to store millis as fraction
            calendar.set(Calendar.MILLISECOND, 0);

            return calendar;
        }
        catch (Throwable e)
        {
            throw new NucleusException("Type java.util.Calendar is not supported without Apache Directory shared-ldap library.");
        }
    }

    public String toString(Object obj)
    {
        String str;
        if (obj instanceof Calendar)
        {
            Calendar calendar = (Calendar) obj;

            // most LDAP servers only store "century year month day hour minute second"
            // TODO: add extension to store millis as fraction
            calendar.set(Calendar.MILLISECOND, 0);

            try
            {
                // this class cannot be linked at compile time since its dependencies are considered as optional
                Class cls = Class.forName("org.apache.directory.shared.ldap.util.GeneralizedTime");

                Constructor constr = ClassUtils.getConstructorWithArguments(cls, new Class[]{Calendar.class});
                Object instance = constr.newInstance(new Object[]{obj});

                Method method = ClassUtils.getMethodForClass(cls, "toGeneralizedTime", null);
                str = (String) method.invoke(instance, (Object[]) null);
            }
            catch (Throwable e)
            {
                throw new NucleusException("Type java.util.Calendar is not supported without Apache Directory shared-ldap library.");
            }
        }
        else
        {
            str = (String) obj;
        }
        return str;
    }

}
